#include <sta/hal/spi.hpp>

#ifdef STA_HAL_SPI_ENABLE

#include <sta/assert.hpp>
#include <sta/endian.hpp>
#include <sta/lang.hpp>


#ifdef STA_MCU_LITTLE_ENDIAN
#  define STA_HAL_SPI_REVERSE_BIT_ORDER SpiBitOrder::MSB
#elif STA_MCU_BIG_ENDIAN
#  define STA_HAL_SPI_REVERSE_BIT_ORDER SpiBitOrder::LSB
#else // !STA_MCU_LITTLE_ENDIAN && !STA_MCU_BIG_ENDIAN
#  ifdef STA_HAL_SPI_REVERSE_BIT_ORDER
#    warning "Internal STA_HAL_SPI_REVERSE_BIT_ORDER macro manually defined! Better now what you are doing!!!"
#  else // !STA_HAL_SPI_REVERSE_BIT_ORDER
#    error "Unknown endian-ness. Define STA_MCU_LITTLE_ENDIAN or STA_MCU_BIG_ENDIAN in <sta/config.hpp>"
# endif // !STA_HAL_SPI_REVERSE_BIT_ORDER
#endif // !STA_MCU_LITTLE_ENDIAN && !STA_MCU_BIG_ENDIAN


namespace sta
{
	static SpiSettings getHalSpiSettings(SPI_HandleTypeDef * handle, uint32_t pclkFreq)
	{
		SpiSettings settings;

		settings.mode        = getSpiMode(
			(handle->Init.CLKPolarity == SPI_POLARITY_LOW) ? SpiClkPolarity::LOW : SpiClkPolarity::HIGH,
			(handle->Init.CLKPhase == SPI_PHASE_1EDGE) ? SpiClkPhase::EDGE_1 : SpiClkPhase::EDGE_2
		);
		settings.dataSize    = (handle->Init.DataSize == SPI_DATASIZE_8BIT) ? SpiDataSize::SIZE_8 : SpiDataSize::SIZE_16;
		settings.bitOrder    = (handle->Init.FirstBit == SPI_FIRSTBIT_MSB) ? SpiBitOrder::MSB : SpiBitOrder::LSB;

		uint32_t prescaler = 1;
		switch (handle->Init.BaudRatePrescaler)
		{
		case SPI_BAUDRATEPRESCALER_2:
			prescaler = 2;
			break;
		case SPI_BAUDRATEPRESCALER_4:
			prescaler = 4;
			break;
		case SPI_BAUDRATEPRESCALER_8:
			prescaler = 8;
			break;
		case SPI_BAUDRATEPRESCALER_16:
			prescaler = 16;
			break;
		case SPI_BAUDRATEPRESCALER_32:
			prescaler = 32;
			break;
		case SPI_BAUDRATEPRESCALER_64:
			prescaler = 64;
			break;
		case SPI_BAUDRATEPRESCALER_128:
			prescaler = 128;
			break;
		case SPI_BAUDRATEPRESCALER_256:
			prescaler = 256;
			break;
		default:
			// Unreachable case
			STA_ASSERT_MSG(false, "Case for SPI_BAUDRATEPRESCALER not handled");
			STA_UNREACHABLE();
		}

		// SPI clock speed is based of PCLK
		settings.clkSpeed = pclkFreq / prescaler;

		return settings;
	}


	HalSpiInterface::HalSpiInterface(const HalSpiInterfaceInfo & info, Mutex * mutex /* = nullptr */)
		: SpiInterface(mutex), info_{info}
	{
		STA_ASSERT(info.handle != nullptr);
		STA_ASSERT(info.getPCLKFreq != nullptr);
	}


	void HalSpiInterface::transfer(uint8_t value)
	{
		if (settings().dataSize == SpiDataSize::SIZE_8)
		{
			HAL_SPI_Transmit(info_.handle, &value, 1, HAL_MAX_DELAY);
		}
		else
		{
			// Required since tx buffer is cast to uint16_t * internally
			uint16_t dummy = value;
			HAL_SPI_Transmit(info_.handle, reinterpret_cast<uint8_t *>(&dummy), 1, HAL_MAX_DELAY);
		}
	}

	void HalSpiInterface::transfer16(uint16_t value)
	{
		static_assert(sizeof(value) == 2, "Unexpected uint16_t size");

		uint16_t size = 1;

		// Send as two bytes if data size is 8-bit
		if (settings().dataSize == SpiDataSize::SIZE_8)
		{
			size = 2;

			if (settings().bitOrder == STA_HAL_SPI_REVERSE_BIT_ORDER)
			{
				// Reverse byte order from internal representation
				value = STA_UINT16_SWAP_BYTE_ORDER(value);
			}
		}

		HAL_SPI_Transmit(info_.handle, reinterpret_cast<uint8_t *>(&value), size, HAL_MAX_DELAY);
	}

	void HalSpiInterface::transfer(const uint8_t * buffer, size_t size)
	{
		STA_ASSERT(buffer != nullptr);
		STA_ASSERT(size != 0);

		HAL_SPI_Transmit(info_.handle, const_cast<uint8_t *>(buffer), size, HAL_MAX_DELAY);
	}

	void HalSpiInterface::transfer(const uint8_t * txBuffer, uint8_t * rxBuffer, size_t size)
	{
		STA_ASSERT(txBuffer != nullptr);
		STA_ASSERT(rxBuffer != nullptr);
		STA_ASSERT(size != 0);

		HAL_SPI_TransmitReceive(info_.handle, const_cast<uint8_t *>(txBuffer), rxBuffer, size, HAL_MAX_DELAY);
	}

	void HalSpiInterface::receive(uint8_t * buffer, size_t size)
	{
		STA_ASSERT(buffer != nullptr);

		HAL_SPI_Receive(info_.handle, buffer, size, HAL_MAX_DELAY);
	}


	void HalSpiInterface::fill(uint8_t value, size_t count)
	{
		STA_ASSERT(count != 0);

		if (settings().dataSize == SpiDataSize::SIZE_8)
		{
			for (size_t i = 0; i < count; ++i)
			{
				HAL_SPI_Transmit(info_.handle, &value, 1, HAL_MAX_DELAY);
			}
		}
		else
		{
			// Required since tx buffer is cast to uint16_t * internally
			uint16_t dummy = value;
			for (size_t i = 0; i < count; ++i)
			{
				HAL_SPI_Transmit(info_.handle, reinterpret_cast<uint8_t *>(&dummy), 1, HAL_MAX_DELAY);
			}
		}
	}


	const SpiSettings & HalSpiInterface::settings() const
	{
		// Cache settings
		static SpiSettings settings = getHalSpiSettings(info_.handle, info_.getPCLKFreq());

		return settings;
	}




	HalSpiDevice::HalSpiDevice(SpiInterface * intf, HalGpioPin csPin)
		: SpiDevice(intf), csPin_{csPin}
	{}

	void HalSpiDevice::select()
	{
		csPin_.setState(GpioPinState::LOW);
	}

	void HalSpiDevice::deselect()
	{
		csPin_.setState(GpioPinState::HIGH);
	}
} // namespace sta


#endif // STA_HAL_SPI_ENABLE
